#!/usr/bin/perl -w
#--------------------------------------------------
# memoSync.pl - Version 1.0
# Demoprogramm, das die Eintrge der
# Merkzettelverwaltung des PalmPilot
# mit einer SQL-Datenbank austauschen kann
#
# (c) 1998, 1999, Ramon Wartala (wartala@gmx.de)
#--------------------------------------------------
use DBI;
use PDA::Pilot;
use Getopt::Std;
use strict;

use vars qw($opt_c $opt_b $opt_a $opt_d $dbh $dlp $found);

my $port      = "/dev/pilot"; # Link auf COM-Port fuer PalmPilot
my $driver    = "mysql";      # Datenbank Treiber
my $database  = "MerkzDB";    # Datenbank Name
my $table     = "Merkzettel"; # Tabellen Name
my $number    = undef;        # Memo Eintrags-Nummer
my $headline  = undef;        # Memo berschrift
my @categories = undef;       # Alle definierten Kategorien
my $category  = undef;        # Ausgewhlte Kategorie
my $text      = undef;        # Memo Text
my $index     = 0;            # Anzahl Datenstze
my $found     = undef;        # markiert gefundenen Kategorie


sub memo_open_pilot {
  my $port = shift;

  # Socket fr Verbindung erzeugen
  my $socket = PDA::Pilot::openPort($port);
  print "HotSync aktivieren...\n";

  # Socket akzeptieren
  $dlp = PDA::Pilot::accept($socket);
}

sub memo_open_sqlDB {
  my ($driver,$database) = @_;
  my $dsn = "DBI:$driver:database=$database";

  # Verbindung zur Datenbank aufbauen
  print "Verbindung mit Datenbank $database aufbauen...\n";
  $dbh = DBI->connect($dsn,undef,undef);
  if(!$dbh) {
    print "Fehler beim Verbindungsaufbau zu $database:".
	"$DBI::errstr\n";
    exit(-1);
  }
}

sub memo_backup {
  my ($port,$driver,$database,$table,$category) = @_;

  # Verbindung zur SQL-Datenbank
  &memo_open_sqlDB($driver,$database);

  # Nachsehen, ob schon eine Tabelle 'Merkzettel' existiert...
  my $rv = $dbh->do("select * from Merkzettel where 0=1");
  if (! $rv) {
    print "Tabelle $table existiert leider noch nicht!\n";
    print "Bitte legen Sie dies mit einem geeigneten Werkzeug an.\n";
    exit(-1);
  }
  &memo_open_pilot($port);

  my $sth = $dbh->prepare('INSERT INTO Merkzettel '.
			  '(Number, Headline, Note,'.
			  'Category, ID) VALUES (?, ?, ?, ?, ?)');

  # Merkzettel datenbank ffnen...
  my $memoDB = $dlp->open("MemoDB");

  # Merzettel-Anwendungsinformationen einlesen
  my $memoApp = $memoDB->getAppBlock;

  # Alle Kategorie-Namen extrahieren und speichern
  @categories = @{$memoApp->{categoryName}};
  
  # Wurde vom Benutzer eine bestimmte Kategorie definiert...
  my $found_category = undef;
  if($category) {
    $found = grep($category eq $_,@categories);
    
    if($found) {
      print "Kategorie $category wird nun ".
	"in Tabelle $table bertragen\n";
      while(my $r = $memoDB->getRecord($index))	{
	  my $r_category = $categories[$r->{category}];
	  if($r_category eq $category) {
	    my $note = $r->{text};
	    my $number = $r->{"index"};
	    my $headline = substr $note,0,26;
	    $headline = $headline.'...';
	    my $id = substr $r->{id},0,9;
	    
	    # In die Datenbank-Tabelle einfgen
	    $sth->execute($number,$headline,$note,$category,$id);
	    die "Fehler: $DBI::err ... $DBI::errstr\n" if $DBI::err;
	    print "Datensatz $index eingefgt...\n";
	  }
	  # Nchster Datensatz
	  $index++;
	} # while
    } else {
      print "Kategorie $category ist nicht gltig\n";
      print "Zur Auswahl stehen nur folgende Kategorien:\n";
      map {print "$_ \n" unless $_ eq ""} @categories;
      exit(-1);
    } # if ($found)
    # wenn nicht, alle Merkzettel aller Kategorien einlesen
  } else {
    # Neuen Merkzettel einlesen
    while(my $r = $memoDB->getRecord($index)) {
      # Datensatz aus MemoDB aufspalten
      my $note = $r->{text};
      my $number = $r->{"index"};
      my $headline = substr $note,0,26;
      my $category = $categories[$r->{category}];
      my $id = substr $r->{id},0,9;
      print $id,"\n";
      
      $headline = $headline.'...';
      
      # In die Datenbank-Tabelle einfgen
      print "Datensatz $index in $table eingefgt\n";
      $sth->execute($number,$headline,$note,$category,$id);
      die "Fehler: $DBI::err ... $DBI::errstr\n" if $DBI::err;
      
      # Nchster Merkzettel
      $index++;
    } # while
  } # if ($category) ... else

  print "MemoDB schlieen\n";
  $memoDB->close;

  # SQL-Statement finalisieren
  $sth->finish;

  print "Verbindung zu Datenbank $database schlieen\n";
  $dbh->disconnect;
}

sub memo_clear {
  my($driver,$database,$table) = @_;

  &memo_open_sqlDB($driver,$database);

  my $sth = $dbh->do('DELETE FROM Merkzettel');
  die "Fehler: $DBI::err ... $DBI::errstr\n" if $DBI::err;
  print "Datenbak Tabelle $table wurde geleert.\n";

  $dbh->disconnect;
}

sub memo_add {
  my ($port,$driver,$database,$table,$text,$category) = @_;

  # Verbindung zur SQL-Datenbank und zum PalmPilot aufnehmen...
  &memo_open_sqlDB($driver,$database);
  &memo_open_pilot($port);

  # Merkzettel Datenbank ffnen
  my $memoDB = $dlp->open("MemoDB");
  my $cat_number = 0;

  if($category) {
    print "Gewhlte Kategorie $category fr neuen Eintrag...";
    my $memoApp = $memoDB->getAppBlock;
    @categories = @{$memoApp->{categoryName}};
    my $found_category = undef;

    foreach $found_category (@categories) {
      if($category eq $found_category) {
        $found = 1;
      }
      $catnumber++;
    }
  }

  # Neuen Datenbankeintrag...
  $index = $DBI::rows;
  $index++;
  my $number = $index;
  my $note = $text;
  my $headline = substr $note,0,26;
  $headline = $headline.'...';

  my $sth = undef;
  my $new_r = PDA::Pilot::MemoRecord->new($note,0,0,$category,0);

  if($found == 1) {
    print "gefunden\n";
    print "Neuen Datensatz einfgen...\n";
    print "\t Nummer: $number\n";
    print "\t berschrift: $headline\n";
    print "\t Text: $note\n";
    print "\t Kategorie: $category ($cat_number)\n";

    $sth = $dbh->prepare('INSERT INTO Merkzettel '.
			 '(Number, Headline, Note, '.
			 'Category, ID) VALUES (?, ?, ?, ?, ?)');
    die "Fehler: $DBI::err ... $DBI::errstr\n" if $DBI::err;
    $sth->execute($number, $headline, $note, $category, undef);

    my $new_r = PDA::Pilot::MemoRecord->new($note,0,0,$cat_number,0);
    $memoDB->setRecord($new_r);
  } else {
    print "nicht gefunden\n" if $category;
    print "Neuer Datensatz als Kategorie \"Nicht abgelegt\" einfgen...\n";
    $sth = $dbh->prepare('INSERT INTO Merkzettel '.
			 '(Number, Headline, Note,' .
			 'Category, ID) VALUES (?, ?, ?, ?, ?)');
    die "Fehler: $DBI::err ... $DBI::errstr\n" if $DBI::err;
    $sth->execute($number, $headline, $note, undef, undef);

    my $new_r = PDA::Pilot::MemoRecord->new($note,0,0,0,0);
    $memoDB->setRecord($new_r);
  }

  $sth->finish;
  $dbh->disconnect;
  $memoDB->close;
  print "fertig!\n";
}

if($#ARGV == -1) {
  print <<USAGE;
  Aufruf: memoSync.pl
  \t-b Backup der PalmPilot MerkzettelDB
  \t-d SQL-Tabelle lschen
  \t-c Kategorie auswhlen
  \t-a Neuen Datensatz

  Beispiel: memoSync.pl -a \"Hallo PalmPilot, hier Perl\" -c\"Privat\"
	     
USAGE
  exit(-1);
} else {
  getopts('bda:c:');

  # Kategorie ausgewhlt?
  if($opt_c) {
    $category = $opt_c;

    if($opt_b) {
      &memo_backup($port,$driver,$database,$table,$category);
    }
    if($opt_a) {
      if((length $opt_a) > 4096) {
        $text = substr $opt_a,0,4095;
      } else {
        $text = $opt_a;
      }
      &memo_add($port,$driver,$database,$table,$text,$category);
    }
    if($opt_d) {
      &memo_clear($driver,$database,$table);
    }
  } else {
    if($opt_b) {
      &memo_backup($port,$driver,$database,$table,undef);
    }
    if($opt_a) {
      if((length $opt_a) > 4096) {
        $text = substr $opt_a,0,4095;
      } else {
        $text = $opt_a;
      }
      &memo_add($port,$driver,$database,$table,$text,undef);
    }
    if($opt_d) {
      &memo_clear($driver,$database,$table,undef);
    }
  }
}


